//==================================================================
//
//	File name	:	label.js
//
//	Summary		:	Label class
//
//	Ceated		:	2018/11/29
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (label) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	label.STYLE_DEFAULT = 0;
	label.STYLE_LINED = 1;
	label.STYLE_INSET = 2;
	label.STYLE_OUTSET = 3;
	label.STYLE_LINK = 4;
	label.STYLE_ICON = 5;
	
	
	//==========================================================
	//
	//	Class		:	Label
	//
	//	Summary		:	Class of label
	//
	//==========================================================
	label.Label = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// ID[String]
					oRect,			// * Size and position[FixedRect]
					iStyle,			// * Label style[int]
					sContent,		// * Content text[string]
					iAlign,			// Text align[int]
					sURL,			// Link URL[string]
					bNewWindow,		// Make new window or not on clicked[boolean]
					bDraggable,		// Draggable or not[boolean]
					fpDragstart)	// Function on start dragging[function]
		{
			// Base class
			super(true, sID, false, oRect, true, -1);
			
			let sTmpID;
			if(sID)
				sTmpID = sID;
			else
				sTmpID = "";
			
			// Check parameters
			if(typeof iStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sTmpID),
											"label.js", "jaswin.label.Label.constructor");
			if((iStyle != label.STYLE_DEFAULT)
			&& (iStyle != label.STYLE_LINED)
			&& (iStyle != label.STYLE_INSET)
			&& (iStyle != label.STYLE_OUTSET)
			&& (iStyle != label.STYLE_LINK)
			&& (iStyle != label.STYLE_ICON))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sTmpID),
											"label.js", "jaswin.label.Label.constructor");
			
			if(typeof sContent != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, sTmpID),
											"label.js", "jaswin.label.Label.constructor");
			
			if(iAlign)
			{
				if(typeof iAlign != "number")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, sTmpID),
											"label.js", "jaswin.label.Label.constructor");
				
				if((iAlign != jaswin.ALIGN_LEFT)
				&& (iAlign != jaswin.ALIGN_RIGHT)
				&& (iAlign != jaswin.ALIGN_CENTER))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_ALIGN, sTmpID),
											"label.js", "jaswin.label.Label.constructor");
			}
			
			if(iStyle == label.STYLE_LINK)
			{
				if(typeof sURL != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_URL, sTmpID),
												"label.js", "jaswin.label.Label.constructor");
				if(bNewWindow)
				{
					if(typeof bNewWindow != "boolean")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_NEW_WINDOW, sTmpID),
													"label.js", "jaswin.label.Label.constructor");
				}
			}
			
			if(bDraggable)
			{
				if(typeof bDraggable != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_DRAGGABLE, sTmpID),
												"label.js", "jaswin.label.Label.constructor");
				if(fpDragstart)
				{
					if(typeof fpDragstart != "function")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_ON_DRAGSTART, sTmpID),
													"label.js", "jaswin.label.Label.constructor");
				}
			}
			
			// Set properties
			this.style = iStyle;					// Label style[int]
			this.align = iAlign;					// Text align[int]
			this.linkURL = sURL;
			this.createWindow = bNewWindow;			// Create new window on clicked[boolean]
			this.draggable = bDraggable;			// Draggable or not[boolean]
			this.onDragstart = fpDragstart;			// Function on dragstart[function(Argument:Label class, event)]
			
			// Create element
			if(this.style == label.STYLE_LINK)
			{
				this.element = document.createElement("a");
				this.element.className = "label";
				this.element.href = "javascript:void(0);";
				if(this.createWindow)
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						window.open(this.linkURL);
					}, false);
				}
				else
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						window.location.href = this.linkURL;
					}, false);
				}
			}
			else
			{
				this.element = document.createElement("span");
				if(this.style == label.STYLE_DEFAULT)
					this.element.className = "label";
				else if(this.style == label.STYLE_LINED)
					this.element.className = "label-lined";
				else if(this.style == label.STYLE_INSET)
					this.element.className = "label-inset";
				else if(this.style == label.STYLE_OUTSET)
					this.element.className = "label-outset";
				else if(this.style == label.STYLE_ICON)
					this.element.className = sContent;
			}
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			
			if(this.style == label.STYLE_DEFAULT)
			{
				this.element.style.width = this.rect.width + "px";
				this.element.style.height = this.rect.height + "px";
			}
			else
			{
				this.element.style.width = this.rect.width - 2 + "px";
				this.element.style.height = this.rect.height - 2 + "px";
			}
			
			if(!this.align)
				this.align = jaswin.ALIGN_LEFT;
			if(this.align == jaswin.ALIGN_LEFT)
				this.element.style.textAlign = "left";
			else if(this.align == jaswin.ALIGN_RIGHT)
				this.element.style.textAlign = "right";
			else if(this.align == jaswin.ALIGN_CENTER)
				this.element.style.textAlign = "center";
			
			if(this.style != label.STYLE_ICON)
				this.element.innerHTML = sContent;
			
			// Event binding
			if(this.draggable)
			{
				this.element.draggable = true;
				this.element.addEventListener("dragstart", this.dragStartEvent = (e) => {
					this.onDragstart(this, e);
				}, false);
			}
		}
		
		//==================================================
		//	Enable this label
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if(this.enabled)
			{
				if(this.style == label.STYLE_LINK)
				{
					if(!this.clickEvent)
					{
						if(this.createWindow)
						{
							this.element.addEventListener("click", this.clickEvent = (e) => {
								window.open(this.linkURL);
							}, false);
						}
						else
						{
							this.element.addEventListener("click", this.clickEvent = (e) => {
								window.location.href = this.linkURL;
							}, false);
						}
					}
					this.element.style.color = "";
				}
				
				if(this.draggable)
				{
					this.element.draggable = true;
					if(!this.dragStartEvent)
					{
						this.element.addEventListener("dragstart", this.dragStartEvent = (e) => {
							this.onDragstart(this, e);
						}, false);
					}
				}
			}
		}
		
		//==================================================
		//	Disable this label
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			if(this.style == label.STYLE_LINK)
			{
				if(this.clickEvent)
				{
					this.element.removeEventListener("click", this.clickEvent, false);
					this.clickEvent = null;
				}
				this.element.style.color = "#808080";
			}
			
			if(this.draggable)
			{
				this.element.draggable = false;
				if(this.dragStartEvent)
				{
					this.element.removeEventListener("dragstart", this.dragStartEvent, false);
					this.dragStartEvent = null;
				}
			}
		}
		
		//==================================================
		//	Get text
		//==================================================
		getText()
		{
			return this.element.textContent;
		}
		
		//==================================================
		//	Set text
		//==================================================
		setText(sText)
		{
			if(this.style == label.STYLE_ICON)
				return;
			
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL),
											"label.js", "jaswin.label.Label.setText");
			
			this.element.innerHTML = sText;
		}
		
		//==================================================
		//	Set font size
		//==================================================
		setFontSize(iFontPoint)
		{
			if(typeof iFontPoint != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_SIZE),
											"label.js", "jaswin.label.Label.setFontSize");
			
			this.element.style.fontSize = iFontPoint + "pt";
		}
		
		//==================================================
		//	Set bold font or not
		//==================================================
		setBold(bBold)
		{
			if(typeof bBold != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BOLD),
											"label.js", "jaswin.label.Label.setBold");
			if(bBold)
				this.element.style.fontWeight = "bold";
			else
				this.element.style.fontWeight = "normal";
		}
		
		//==================================================
		//	Set font color
		//==================================================
		setFontColor(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_COLOR),
											"label.js", "jaswin.label.Label.setFontColor");
			this.element.style.color = sColor;
		}
		
		//==================================================
		//	Set font family
		//==================================================
		setFontFamily(sFontFamily)
		{
			if(typeof sFontFamily != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_FAMILY),
											"label.js", "jaswin.label.Label.setFontFamily");
			this.element.style.fontFamily = sFontFamily;
		}
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND),
											"label.js", "jaswin.label.Label.setBackgroundColor");
			this.element.style.backgroundColor = sColor;
		}
		
		//==================================================
		//	Set border color
		//==================================================
		setBorderColor(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BORDERCOLOR),
											"label.js", "jaswin.label.Label.setBorderColor");
			this.element.style.borderColor = sColor;
		}
	}
}(jaswin.label = jaswin.label || {}));
